import { CharUtil } from './charstream';
import CharStream = CharUtil.CharStream;
export declare namespace Primitives {
    let PAD: number;
    class EOFMark {
        private static _instance;
        private constructor();
        static get Instance(): EOFMark;
    }
    const EOF: EOFMark;
    /**
     * Represents a successful parse.
     */
    class Success<T> {
        tag: 'success';
        inputstream: CharStream;
        result: T;
        /**
         * Returns an object representing a successful parse.
         * @param istream The remaining string.
         * @param res The result of the parse
         */
        constructor(istream: CharStream, res: T);
    }
    /**
     * Represents a failed parse.
     */
    class Failure {
        tag: 'failure';
        inputstream: CharStream;
        error_pos: number;
        error_msg: string;
        is_critical: boolean;
        /**
         * Returns an object representing a failed parse.
         * If the failure is critical, then parsing will stop immediately.
         *
         * @param istream The string, unmodified, that was given to the parser.
         * @param error_pos The position of the parsing failure in istream.
         * @param error_msg The error message for the failure.
         * @param is_critical Whether or not the failure should cause an enclosing choice to fail (i.e., short-circuit).
         */
        constructor(istream: CharStream, error_pos: number, error_msg?: string, is_critical?: boolean);
    }
    /**
     * Union type representing a successful or failed parse.
     */
    type Outcome<T> = Success<T> | Failure;
    /**
     * Generic type of a parser.
     */
    interface IParser<T> {
        (inputstream: CharStream): Generator<any, Outcome<T>, undefined>;
    }
    /**
     * result succeeds without consuming any input, and returns v.
     * @param v The result of the parse.
     */
    function result<T>(v: T): IParser<T>;
    /**
     * This data structure represents a mutable "cell" for
     * separating declarations from implementations ala
     * the recparser from FParsec.
     */
    interface RefCell<T> {
        contents: IParser<T>;
    }
    /**
     * This data structure represents a mutable "cell" for
     * separating declarations from implementations ala
     * the recparser from FParsec. The difference between
     * Arg1RefCell and RefCell is that the former can
     * store parser functions that take an argument.
     */
    interface Arg1RefCell<A, T> {
        contents: (arg: A) => IParser<T>;
    }
    /**
     * `recParser` is a forward declaration for a recursive parser.
     * It is effectively a form of deferred evaluation to prevent
     * Javascript from eagerly expanding recursive grammar productions.
     * @returns A pair, `[decl,impl]` where `decl` is a parser declaration and `impl` is an implementation for p
     */
    function recParser<T>(): [IParser<T>, RefCell<T>];
    /**
     * `rec1ArgParser` is a forward declaration for a recursive parser that
     * takes one argument. It is effectively a form of deferred evaluation to
     * prevent Javascript from eagerly expanding recursive grammar productions.
     * @returns A pair, `[decl,impl]` where `decl` is a parser declaration and `impl` is an implementation for p
     */
    function rec1ArgParser<A, T>(): [(arg: A) => IParser<T>, Arg1RefCell<A, T>];
    /**
     * zero fails without consuming any input.
     * @param msg the error message.
     */
    function zero<T>(msg: string): IParser<T>;
    /**
     * `fail` fails if the given parser `p` succeeds.  On failure,
     * `fail` returns the given error message.  For either outcome,
     * success or failure, `fail` never consumes input.
     * @param p A parser
     * @param msg An error message
     * @returns
     */
    function fail<T>(p: IParser<T>): (msg: string) => IParser<undefined>;
    /**
     * `ok` succeeds without consuming any input, returning whatever
     * is given here.
     * @param res A result object.
     */
    function ok<T>(res: T): IParser<T>;
    /**
     * expect tries to apply the given parser and returns the result of that parser
     * if it succeeds, otherwise it returns a critical Failure
     * If the parser results in a critical Failure, expect simply returns it,
     * otherwise expect creates a critical Failure with the given error message
     * and the start pos of the istream as the error pos.
     *
     * @param parser The parser to try
     * @param error_msg The error message if the parser fails
     */
    function expect<T>(parser: IParser<T>): (error_msg: string) => (istream: CharStream) => Generator<any, Failure | Success<T>, undefined>;
    /**
     * item successfully consumes the first character if the input
     * string is non-empty, otherwise it fails.
     */
    const item: IParser<CharStream>;
    /**
     * bind is a curried function that takes a parser p and returns
     * a function that takes a parser f which returns the composition
     * of p and f.  If _any_ of the parsers fail, the original inputstream
     * is returned in the Failure object (i.e., bind backtracks).
     * @param p A parser
     */
    function bind<T, U>(p: IParser<T>): (f: (t: T) => IParser<U>) => (istream: CharStream) => Generator<any, Failure | Success<U>, undefined>;
    function delay<T>(p: IParser<T>): () => IParser<T>;
    /**
     * `seq` takes a parser `p` and a parser `q`. It applies `p` to the input,
     * passing the remaining input stream to `q`; `q` is then applied. It
     * returns the result of `p` and `q` as a tuple.
     * @param p A parser
     */
    function seq<T, U>(p: IParser<T>): (q: IParser<U>) => (istream: CharUtil.CharStream) => Generator<any, Failure | Success<[T, U]>, undefined>;
    /**
     * `sat` takes a predicate and yields a parser that consumes a
     * single character if the character satisfies the predicate,
     * otherwise it fails.
     */
    function sat(p: (ch: string) => boolean): IParser<CharStream>;
    /**
     * `satClass` takes an array of satisfactory characters and yields
     * a parser that consumes a single character if the character
     * is in the array, otherwise it fails.
     */
    function satClass(char_class: string[]): IParser<CharStream>;
    /**
     * char takes a character and yields a parser that consume
     * that character. The returned parser succeeds if the next
     * character in the input stream is c, otherwise it fails.
     * @param c
     */
    function char(c: string): IParser<CharStream>;
    const lower_chars: string[];
    const upper_chars: string[];
    /**
     * letter returns a parser that consumes a single alphabetic
     * character, from a-z, regardless of case.
     */
    const letter: IParser<CharStream>;
    /**
     * digit returns a parser that consumes a single numeric
     * character, from 0-9.  Note that the type of the result
     * is a string, not a number.
     */
    const digit: IParser<CharStream>;
    /**
     * An integer parser.
     */
    const integer: IParser<number>;
    /**
     * A floating point parser, with optional fraction.
     */
    const float: IParser<number>;
    /**
     * upper returns a parser that consumes a single character
     * if that character is uppercase.
     */
    const upper: IParser<CharStream>;
    /**
     * lower returns a parser that consumes a single character
     * if that character is lowercase.
     */
    const lower: IParser<CharStream>;
    /**
     * choice specifies an ordered choice between two parsers,
     * p1 and p2. The returned parser will first apply
     * parser p1.  If p1 succeeds, p1's Outcome is returned.
     * If p1 fails, p2 is applied and the Outcome of p2 is returned.
     *
     * An exception is when an outcome is a critical failure,
     * that outcome is immediately returned.
     *
     * @param p1 A parser.
     */
    function choice<T>(p1: IParser<T>): (p2: IParser<T>) => IParser<T>;
    /**
     * Like choice, but chooses from multiple possible parsers
     * The parser will be tried in the order of the input, and the result of
     * the first parser to succeed is returned
     * Example usage: choices(p1, p2, p3)
     *
     * @param parsers An array of parsers to try
     */
    function choices<T>(...parsers: IParser<T>[]): IParser<T>;
    /**
     * `prefix` is a prefix-optimized `choice` combinator.  If `pre`
     * succeeds, `p` is called, and `prefix`'s and `p`'s results
     * are returned by calling the reducer `f`.  If `p` fails,
     * just `p`'s result is returned. Use this whereever you would
     * write choice(p)(pipe2(p)(q)(f)).
     * @param pre A prefix parser that should always succeed.
     * @param p A suffix parser that may succeed.
     * @param f A function that is called only if `p` succeeds.
     */
    function prefix<T, U>(pre: IParser<T>): (p: IParser<U>) => (f: (t: T, u: U) => T) => (istream: CharStream) => Generator<any, Failure | Success<T>, undefined>;
    /**
     * appfun allows the user to apply a function f to
     * the result of a parser p, assuming that p is successful.
     * This is the same as the `|>>`
     * function from FParsec and is an alias for `pipe`.
     * @param p A parser.
     */
    function appfun<T, U>(p: IParser<T>): (f: (t: T) => U) => (istream: CharStream) => Generator<any, Failure | Success<U>, undefined>;
    /**
     * `pipe` allows the user to apply a function f to
     * the result of a parser p, assuming that p is successful.
     * This is the same as the `|>>`
     * function from FParsec and is an alias for `appfun`
     * @param p A parser.
     */
    const pipe: typeof appfun;
    /**
     * `pipe2(p1)(p2)(f)` applies the parsers `p1` and `p2` in sequence.
     * It returns the result of the function application `f(t,u)`, where
     * `t` and `u` are the results returned by `p1` and `p2`.
     * @param p1 A parser.
     * @param p2 Another parser.
     * @param f A function that takes the result of `p1` and `p2`.
     */
    function pipe2<T, U, V>(p: IParser<T>): (q: IParser<U>) => (f: (t: T, u: U) => V) => (istream: CharUtil.CharStream) => Generator<any, Failure | Success<V>, undefined>;
    /**
     * `pipe3(p1)(p2)(p3)(f)` applies the parsers `p1`, `p2`, and `p3` in
     * sequence. It returns the result of the function application `f(a,b,c)`,
     * where `a`, `b`, and `c` are the results returned by `p1`, `p2`, and
     * `p3`, respectively.
     * @param p1 A parser.
     * @param p2 A parser.
     * @param p3 A parser.
     * @param f A function that takes the results of `p1`, `p2`, and `p3`.
     */
    function pipe3<A, B, C, D>(p1: IParser<A>): (p2: IParser<B>) => (p3: IParser<C>) => (f: (a: A, b: B, c: C) => D) => (istream: CharUtil.CharStream) => Generator<any, Failure | Success<D>, undefined>;
    /**
     * many repeatedly applies the parser p until p fails. many always
     * succeeds, even if it matches nothing or if an outcome is critical.
     * many tries to guard against an infinite loop by raising an exception
     * if p succeeds without changing the parser state.
     * @param p The parser to try
     */
    function many<T>(p: IParser<T>): IParser<T[]>;
    /**
     * many1 repeatedly applies the parser p until p fails. many1 must
     * succeed at least once.  many1 tries to guard against an infinite
     * loop by raising an exception if p succeeds without changing the
     * parser state.
     * @param p The parser to try
     */
    function many1<T>(p: IParser<T>): (istream: CharStream) => Generator<any, Failure | Success<T[]>, undefined>;
    /**
     * `str` yields a parser for the given string.
     * @param s A string
     */
    function str(s: string): IParser<CharStream>;
    /**
     * Returns a parser that succeeds only if the end of the
     * input has been reached.
     */
    const eof: IParser<EOFMark>;
    /**
     * fresult returns a parser that applies the parser p,
     * and if p succeeds, returns the value x.
     * @param p a parser
     */
    function fresult<T, U>(p: IParser<T>): (x: U) => (istream: CharStream) => Generator<any, Failure | Success<U>, undefined>;
    /**
     * left returns a parser that applies the parser p,
     * then the parser q, and if both are successful,
     * returns the result of p.
     * @param p a parser
     */
    function left<T, U>(p: IParser<T>): (q: IParser<U>) => (istream: CharStream) => Generator<any, Failure | Success<T>, undefined>;
    /**
     * right returns a parser that applies the parser p,
     * then the parser q, and if both are successful,
     * returns the result of q.
     * @param p a parser
     */
    function right<T, U>(p: IParser<T>): (q: IParser<U>) => (istream: CharStream) => Generator<any, Failure | Success<U>, undefined>;
    /**
     * between returns a parser that applies the parser
     * popen, p, and pclose in sequence, and if all are
     * successful, returns the result of p.
     * @param popen the first parser
     */
    function between<T, U, V>(popen: IParser<T>): (pclose: IParser<U>) => (p: IParser<V>) => IParser<V>;
    /**
     * The debug parser takes a parser p and a debug string,
     * printing the debug string as a side-effect before
     * applying p to the input.
     * @param p a parser
     */
    function debug<T>(p: IParser<T>): (label: string) => (istream: CharStream) => Generator<any, Outcome<T>, undefined>;
    /**
     * Produce a diagnostic message for a parser failure.
     * @param fail The Failure object.
     * @param windowSz The amount of context (in chars) to show to the left and right of the failure position.
     */
    function diagnosticMessage(fail: Failure, windowSz: number): string;
    /**
     * nl matches and returns a newline.
     */
    const nl: IParser<CharStream>;
    /**
     * ws matches zero or more of the following whitespace characters:
     * ' ', '\t', '\n', or '\r\n'
     * ws returns matched whitespace in a single CharStream result.
     * Note: ws NEVER fails
     */
    const ws: IParser<CharStream>;
    /**
     * ws1 matches one or more of the following whitespace characters:
     * ' ', '\t', '\n', or '\r\n'
     * ws1 returns matched whitespace in a single CharStream result.
     */
    const ws1: IParser<CharStream>;
    /**
     * Match any of given alternatives in the given array of strings. Matches
     * longest-first and, where length is the same, lexicographically first.
     * @param strs An array of acceptable strings.
     */
    function strSat(strs: string[]): IParser<CharStream>;
    /**
     * An optimized parser that seeks the input until the given predicate does not
     * return true.  On success, it returns the matching CharStream.
     * @param pred A function that returns true if the given character should be accepted.
     */
    function matchWhile(pred: (char: string) => boolean): IParser<CharStream>;
    /**
     * An optimized parser that seeks the input until the given predicate does not
     * return true.  On success, it returns the matching CharStream.
     * @param pred A function that returns true if the given character code should be accepted.
     */
    function matchWhileCharCode(pred: (char: number) => boolean): IParser<CharStream>;
}
