"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Primitives = void 0;
const charstream_1 = require("./charstream");
var CharStream = charstream_1.CharUtil.CharStream;
var Primitives;
(function (Primitives) {
    Primitives.PAD = 10; // PAD*2 is the max diagnostic message size
    class EOFMark {
        constructor() { }
        static get Instance() {
            return this._instance || (this._instance = new this());
        }
    }
    Primitives.EOFMark = EOFMark;
    Primitives.EOF = EOFMark.Instance;
    /**
     * Represents a successful parse.
     */
    class Success {
        /**
         * Returns an object representing a successful parse.
         * @param istream The remaining string.
         * @param res The result of the parse
         */
        constructor(istream, res) {
            this.tag = 'success';
            this.inputstream = istream;
            this.result = res;
        }
    }
    Primitives.Success = Success;
    /**
     * Represents a failed parse.
     */
    class Failure {
        /**
         * Returns an object representing a failed parse.
         * If the failure is critical, then parsing will stop immediately.
         *
         * @param istream The string, unmodified, that was given to the parser.
         * @param error_pos The position of the parsing failure in istream.
         * @param error_msg The error message for the failure.
         * @param is_critical Whether or not the failure should cause an enclosing choice to fail (i.e., short-circuit).
         */
        constructor(istream, error_pos, error_msg = '', is_critical = false) {
            this.tag = 'failure';
            this.inputstream = istream;
            this.error_pos = error_pos;
            this.error_msg = error_msg;
            this.is_critical = is_critical;
        }
    }
    Primitives.Failure = Failure;
    /**
     * result succeeds without consuming any input, and returns v.
     * @param v The result of the parse.
     */
    function result(v) {
        return function* (istream) {
            return new Success(istream, v);
        };
    }
    Primitives.result = result;
    /**
     * `recParser` is a forward declaration for a recursive parser.
     * It is effectively a form of deferred evaluation to prevent
     * Javascript from eagerly expanding recursive grammar productions.
     * @returns A pair, `[decl,impl]` where `decl` is a parser declaration and `impl` is an implementation for p
     */
    function recParser() {
        const dumbParser = (_input) => {
            throw new Error('You forgot to initialize your recursive parser.');
        };
        const r = { contents: dumbParser };
        const p = (input) => r.contents(input);
        return [p, r];
    }
    Primitives.recParser = recParser;
    /**
     * `rec1ArgParser` is a forward declaration for a recursive parser that
     * takes one argument. It is effectively a form of deferred evaluation to
     * prevent Javascript from eagerly expanding recursive grammar productions.
     * @returns A pair, `[decl,impl]` where `decl` is a parser declaration and `impl` is an implementation for p
     */
    function rec1ArgParser() {
        const dumbParser = (_arg) => (_input) => {
            throw new Error('You forgot to initialize your recursive parser.');
        };
        const r = { contents: dumbParser };
        const p = (arg) => (input) => r.contents(arg)(input);
        return [p, r];
    }
    Primitives.rec1ArgParser = rec1ArgParser;
    /**
     * zero fails without consuming any input.
     * @param msg the error message.
     */
    function zero(msg) {
        return function* (istream) {
            return new Failure(istream, istream.startpos, msg);
        };
    }
    Primitives.zero = zero;
    /**
     * `fail` fails if the given parser `p` succeeds.  On failure,
     * `fail` returns the given error message.  For either outcome,
     * success or failure, `fail` never consumes input.
     * @param p A parser
     * @param msg An error message
     * @returns
     */
    function fail(p) {
        return function (msg) {
            return function* (istream) {
                const o = yield* p(istream);
                switch (o.tag) {
                    case 'success':
                        return new Failure(istream, istream.startpos, msg, false);
                    case 'failure':
                        return new Success(istream, undefined);
                }
            };
        };
    }
    Primitives.fail = fail;
    /**
     * `ok` succeeds without consuming any input, returning whatever
     * is given here.
     * @param res A result object.
     */
    function ok(res) {
        return function* (istream) {
            return new Success(istream, res);
        };
    }
    Primitives.ok = ok;
    /**
     * expect tries to apply the given parser and returns the result of that parser
     * if it succeeds, otherwise it returns a critical Failure
     * If the parser results in a critical Failure, expect simply returns it,
     * otherwise expect creates a critical Failure with the given error message
     * and the start pos of the istream as the error pos.
     *
     * @param parser The parser to try
     * @param error_msg The error message if the parser fails
     */
    function expect(parser) {
        return function (error_msg) {
            return function* (istream) {
                const outcome = yield* parser(istream);
                switch (outcome.tag) {
                    case 'success':
                        return outcome;
                    case 'failure':
                        return outcome.is_critical ? outcome : new Failure(istream, istream.startpos, error_msg, true);
                }
            };
        };
    }
    Primitives.expect = expect;
    /**
     * item successfully consumes the first character if the input
     * string is non-empty, otherwise it fails.
     */
    Primitives.item = function* (istream) {
        if (istream.isEmpty()) {
            return new Failure(istream, istream.startpos);
        }
        else {
            const remaining = istream.tail(); // remaining string;
            const res = istream.head(); // result of parse;
            return new Success(remaining, res);
        }
    };
    /**
     * bind is a curried function that takes a parser p and returns
     * a function that takes a parser f which returns the composition
     * of p and f.  If _any_ of the parsers fail, the original inputstream
     * is returned in the Failure object (i.e., bind backtracks).
     * @param p A parser
     */
    function bind(p) {
        return function (f) {
            return function* (istream) {
                const r = yield* p(istream);
                switch (r.tag) {
                    case 'success':
                        const o = yield* f(r.result)(r.inputstream);
                        switch (o.tag) {
                            case 'success':
                                break;
                            case 'failure': // note: backtracks, returning original istream
                                return new Failure(istream, o.error_pos, o.error_msg, o.is_critical);
                        }
                        return o;
                    case 'failure':
                        return new Failure(istream, r.error_pos, r.error_msg, r.is_critical);
                }
            };
        };
    }
    Primitives.bind = bind;
    function delay(p) {
        return () => p;
    }
    Primitives.delay = delay;
    /**
     * `seq` takes a parser `p` and a parser `q`. It applies `p` to the input,
     * passing the remaining input stream to `q`; `q` is then applied. It
     * returns the result of `p` and `q` as a tuple.
     * @param p A parser
     */
    function seq(p) {
        return (q) => {
            return bind(p)(t => {
                return bind(q)(u => {
                    return result([t, u]);
                });
            });
        };
    }
    Primitives.seq = seq;
    /**
     * `sat` takes a predicate and yields a parser that consumes a
     * single character if the character satisfies the predicate,
     * otherwise it fails.
     */
    function sat(p) {
        const f = function (x) {
            const char = x.toString();
            if (char.length !== 1)
                throw new Error('Input to predicate must be a character.');
            return p(char)
                ? result(x)
                : function* (istream) {
                    return new Failure(istream, istream.startpos - 1);
                };
        };
        return bind(Primitives.item)(f);
    }
    Primitives.sat = sat;
    /**
     * `satClass` takes an array of satisfactory characters and yields
     * a parser that consumes a single character if the character
     * is in the array, otherwise it fails.
     */
    function satClass(char_class) {
        const f = (x) => {
            return char_class.indexOf(x.toString()) > -1
                ? result(x)
                : function* (istream) {
                    return new Failure(istream, istream.startpos - 1);
                };
        };
        return bind(Primitives.item)(f);
    }
    Primitives.satClass = satClass;
    /**
     * char takes a character and yields a parser that consume
     * that character. The returned parser succeeds if the next
     * character in the input stream is c, otherwise it fails.
     * @param c
     */
    function char(c) {
        if (c.length != 1) {
            throw new Error('char parser takes a string of length 1 (i.e., a char)');
        }
        return satClass([c]);
    }
    Primitives.char = char;
    Primitives.lower_chars = 'abcdefghijklmnopqrstuvwxyz'.split('');
    Primitives.upper_chars = 'abcdefghijklmnopqrstuvwxyz'.toUpperCase().split('');
    /**
     * letter returns a parser that consumes a single alphabetic
     * character, from a-z, regardless of case.
     */
    Primitives.letter = satClass(Primitives.lower_chars.concat(Primitives.upper_chars));
    /**
     * digit returns a parser that consumes a single numeric
     * character, from 0-9.  Note that the type of the result
     * is a string, not a number.
     */
    Primitives.digit = satClass(['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']);
    /**
     * An integer parser.
     */
    Primitives.integer = appfun(many1(Primitives.digit))(arr => {
        const s = CharStream.concat(arr).toString();
        return parseFloat(s);
    });
    /**
     * A floating point parser, with optional fraction.
     */
    Primitives.float = choice(pipe2(Primitives.integer)(right(char('.'))(Primitives.integer))((a, b) => parseFloat(a.toString() + '.' + b.toString())))(Primitives.integer);
    /**
     * upper returns a parser that consumes a single character
     * if that character is uppercase.
     */
    Primitives.upper = satClass(Primitives.upper_chars);
    /**
     * lower returns a parser that consumes a single character
     * if that character is lowercase.
     */
    Primitives.lower = satClass(Primitives.lower_chars);
    /**
     * choice specifies an ordered choice between two parsers,
     * p1 and p2. The returned parser will first apply
     * parser p1.  If p1 succeeds, p1's Outcome is returned.
     * If p1 fails, p2 is applied and the Outcome of p2 is returned.
     *
     * An exception is when an outcome is a critical failure,
     * that outcome is immediately returned.
     *
     * @param p1 A parser.
     */
    function choice(p1) {
        return (p2) => {
            return function* (istream) {
                const o = yield* p1(istream);
                switch (o.tag) {
                    case 'success':
                        return o;
                    case 'failure':
                        if (o.is_critical) {
                            return o;
                        }
                        const o2 = yield* p2(istream);
                        switch (o2.tag) {
                            case 'success':
                                break;
                            case 'failure':
                                return o2.is_critical || o2.error_pos >= o.error_pos ? o2 : o;
                        }
                        return o2;
                }
            };
        };
    }
    Primitives.choice = choice;
    /**
     * Like choice, but chooses from multiple possible parsers
     * The parser will be tried in the order of the input, and the result of
     * the first parser to succeed is returned
     * Example usage: choices(p1, p2, p3)
     *
     * @param parsers An array of parsers to try
     */
    function choices(...parsers) {
        if (parsers.length == 0) {
            throw new Error('Error: choices must have a non-empty array.');
        }
        return function* (istream) {
            let i = 0;
            while (true && i < parsers.length) {
                const o = yield* parsers[i](istream);
                if (o.tag === 'success') {
                    return o;
                }
                else if (o.is_critical) {
                    return o;
                }
                i++;
            }
            return new Failure(istream, istream.startpos);
        };
    }
    Primitives.choices = choices;
    /**
     * `prefix` is a prefix-optimized `choice` combinator.  If `pre`
     * succeeds, `p` is called, and `prefix`'s and `p`'s results
     * are returned by calling the reducer `f`.  If `p` fails,
     * just `p`'s result is returned. Use this whereever you would
     * write choice(p)(pipe2(p)(q)(f)).
     * @param pre A prefix parser that should always succeed.
     * @param p A suffix parser that may succeed.
     * @param f A function that is called only if `p` succeeds.
     */
    function prefix(pre) {
        return (p) => {
            return (f) => {
                return function* (istream) {
                    const output1 = yield* pre(istream);
                    switch (output1.tag) {
                        case 'success':
                            const output2 = yield* p(output1.inputstream);
                            switch (output2.tag) {
                                case 'success':
                                    return new Success(output2.inputstream, f(output1.result, output2.result));
                                case 'failure':
                                    return output1;
                            }
                        case 'failure':
                            return output1;
                    }
                };
            };
        };
    }
    Primitives.prefix = prefix;
    /**
     * appfun allows the user to apply a function f to
     * the result of a parser p, assuming that p is successful.
     * This is the same as the `|>>`
     * function from FParsec and is an alias for `pipe`.
     * @param p A parser.
     */
    function appfun(p) {
        return (f) => {
            return function* (istream) {
                const o = yield* p(istream);
                switch (o.tag) {
                    case 'success':
                        return new Success(o.inputstream, f(o.result));
                    case 'failure':
                        return o;
                }
            };
        };
    }
    Primitives.appfun = appfun;
    /**
     * `pipe` allows the user to apply a function f to
     * the result of a parser p, assuming that p is successful.
     * This is the same as the `|>>`
     * function from FParsec and is an alias for `appfun`
     * @param p A parser.
     */
    Primitives.pipe = appfun;
    /**
     * `pipe2(p1)(p2)(f)` applies the parsers `p1` and `p2` in sequence.
     * It returns the result of the function application `f(t,u)`, where
     * `t` and `u` are the results returned by `p1` and `p2`.
     * @param p1 A parser.
     * @param p2 Another parser.
     * @param f A function that takes the result of `p1` and `p2`.
     */
    function pipe2(p) {
        return (q) => {
            return (f) => {
                return bind(p)(t => {
                    return bind(q)(u => {
                        return result(f(t, u));
                    });
                });
            };
        };
    }
    Primitives.pipe2 = pipe2;
    /**
     * `pipe3(p1)(p2)(p3)(f)` applies the parsers `p1`, `p2`, and `p3` in
     * sequence. It returns the result of the function application `f(a,b,c)`,
     * where `a`, `b`, and `c` are the results returned by `p1`, `p2`, and
     * `p3`, respectively.
     * @param p1 A parser.
     * @param p2 A parser.
     * @param p3 A parser.
     * @param f A function that takes the results of `p1`, `p2`, and `p3`.
     */
    function pipe3(p1) {
        return (p2) => {
            return (p3) => {
                return (f) => {
                    return pipe2(
                    // parse p1
                    p1)(pipe2(
                    // then parse p2
                    p2)(
                    // then parse p3
                    p3)(
                    // then return a tuple (b,c)
                    (b, c) => [b, c]))(
                    // then apply f to all of a, b, c
                    (a, [b, c]) => f(a, b, c));
                };
            };
        };
    }
    Primitives.pipe3 = pipe3;
    /**
     * many repeatedly applies the parser p until p fails. many always
     * succeeds, even if it matches nothing or if an outcome is critical.
     * many tries to guard against an infinite loop by raising an exception
     * if p succeeds without changing the parser state.
     * @param p The parser to try
     */
    function many(p) {
        return function* (istream) {
            let istream2 = istream;
            const outputs = [];
            let succeeds = true;
            while (!istream2.isEmpty() && succeeds) {
                const o = yield* p(istream2);
                switch (o.tag) {
                    case 'success':
                        if (istream2 == o.inputstream) {
                            throw new Error('Parser loops infinitely.');
                        }
                        istream2 = o.inputstream;
                        outputs.push(o.result);
                        break;
                    case 'failure':
                        succeeds = false;
                        break;
                }
            }
            return new Success(istream2, outputs);
        };
    }
    Primitives.many = many;
    /**
     * many1 repeatedly applies the parser p until p fails. many1 must
     * succeed at least once.  many1 tries to guard against an infinite
     * loop by raising an exception if p succeeds without changing the
     * parser state.
     * @param p The parser to try
     */
    function many1(p) {
        return (istream) => {
            return pipe2(p)(many(p))((hd, tl) => {
                tl.unshift(hd);
                return tl;
            })(istream);
        };
    }
    Primitives.many1 = many1;
    /**
     * `str` yields a parser for the given string.
     * @param s A string
     */
    function str(s) {
        return function* (istream) {
            if (istream.peekMatches(s)) {
                return new Success(istream.seek(s.length), new CharStream(s));
            }
            else {
                return new Failure(istream, istream.startpos);
            }
        };
    }
    Primitives.str = str;
    /**
     * Returns a parser that succeeds only if the end of the
     * input has been reached.
     */
    Primitives.eof = function* (istream) {
        return istream.isEOF() ? new Success(istream, Primitives.EOF) : new Failure(istream, istream.startpos);
    };
    /**
     * fresult returns a parser that applies the parser p,
     * and if p succeeds, returns the value x.
     * @param p a parser
     */
    function fresult(p) {
        return (x) => {
            return (istream) => {
                return bind(p)((_t) => result(x))(istream);
            };
        };
    }
    Primitives.fresult = fresult;
    /**
     * left returns a parser that applies the parser p,
     * then the parser q, and if both are successful,
     * returns the result of p.
     * @param p a parser
     */
    function left(p) {
        return (q) => {
            return (istream) => {
                return bind(p)((t) => fresult(q)(t))(istream);
            };
        };
    }
    Primitives.left = left;
    /**
     * right returns a parser that applies the parser p,
     * then the parser q, and if both are successful,
     * returns the result of q.
     * @param p a parser
     */
    function right(p) {
        return (q) => {
            return (istream) => {
                return bind(p)(_ => q)(istream);
            };
        };
    }
    Primitives.right = right;
    /**
     * between returns a parser that applies the parser
     * popen, p, and pclose in sequence, and if all are
     * successful, returns the result of p.
     * @param popen the first parser
     */
    function between(popen) {
        return (pclose) => {
            return (p) => {
                const l = left(p)(pclose);
                const r = right(popen)(l);
                return r;
            };
        };
    }
    Primitives.between = between;
    /**
     * The debug parser takes a parser p and a debug string,
     * printing the debug string as a side-effect before
     * applying p to the input.
     * @param p a parser
     */
    function debug(p) {
        return (label) => {
            return function* (istream) {
                console.log('apply: ' + label + ', startpos: ' + istream.startpos + ', endpos: ' + istream.endpos);
                const o = yield* p(istream);
                switch (o.tag) {
                    case 'success':
                        console.log('success: ' + label + ', startpos: ' + istream.startpos + ', endpos: ' + istream.endpos);
                        break;
                    case 'failure':
                        console.log('failure: ' +
                            label +
                            ', startpos: ' +
                            istream.startpos +
                            ', endpos: ' +
                            istream.endpos +
                            '\n' +
                            diagnosticMessage(o, Primitives.PAD));
                        break;
                }
                return o;
            };
        };
    }
    Primitives.debug = debug;
    /**
     * Get the index of the left side of the input stream error window.
     * @param windowSz The size of the error window.
     * @param failurePos The position of the error in the input stream.
     * @returns An index.
     */
    function windowLeftIndex(windowSz, failurePos) {
        return failurePos - windowSz < 0 ? 0 : failurePos - windowSz;
    }
    /**
     * Get the index of the right side of the input stream error window.
     * @param windowSz The size of the error window.
     * @param failurePos The position of the error in the input stream.
     * @param bufferLen The total length of the input stream.
     * @returns An index.
     */
    function windowRightIndex(windowSz, failurePos, bufferLen) {
        return failurePos + windowSz >= bufferLen ? bufferLen - 1 : failurePos + windowSz;
    }
    /**
     * Finds the index of the newline closest to the failure position in the left side of the input window.
     * @param leftIndex The left bound of the input window.
     * @param failurePos The position of the error in the input stream.
     * @param buffer The input.
     * @returns An index.
     */
    function indexOfLastNewlineLeftWindow(leftIndex, failurePos, buffer) {
        function searchBackward(pos) {
            if (pos <= leftIndex) {
                return -1;
            }
            else if (buffer[pos] === '\n') {
                return pos;
            }
            else {
                return searchBackward(pos - 1);
            }
        }
        const idx = searchBackward(failurePos - 1);
        return idx === -1 ? leftIndex : idx;
    }
    /**
     * Finds the index of the newline closest to the failure position in the right side of the input window.
     * @param rightIndex The right bound of the input window.
     * @param failurePos The position of the error in the input stream.
     * @param buffer The input.
     * @returns An index.
     */
    function indexOfFirstNewlineRightWindow(rightIndex, failurePos, buffer) {
        function searchForward(pos) {
            if (pos >= rightIndex) {
                return -1;
            }
            else if (buffer[pos] === '\n') {
                return pos;
            }
            else {
                return searchForward(pos + 1);
            }
        }
        const idx = searchForward(failurePos - 1);
        return idx === -1 ? rightIndex : idx;
    }
    /**
     * Pads a string `s` with `padStr` `num` times.
     * @param s
     * @param padStr
     * @param num
     * @returns
     */
    function leftPad(s, padStr, num) {
        return num > 0 ? leftPad(padStr + s, padStr, num - 1) : s;
    }
    /**
     * Produce a diagnostic message for a parser failure.
     * @param fail The Failure object.
     * @param windowSz The amount of context (in chars) to show to the left and right of the failure position.
     */
    function diagnosticMessage(fail, windowSz) {
        const failurePos = fail.error_pos;
        const buffer = fail.inputstream.input.toString();
        const err = fail.error_msg;
        // compute window
        const leftIdx = windowLeftIndex(windowSz, failurePos);
        const rightIdx = windowRightIndex(windowSz, failurePos, buffer.length);
        const afterLastNLLeft = indexOfLastNewlineLeftWindow(leftIdx, failurePos, buffer) + 1;
        const beforeFirstNLRight = indexOfFirstNewlineRightWindow(rightIdx, failurePos, buffer) - 1;
        const winLeftIdx = Math.max(leftIdx, afterLastNLLeft);
        const winRightIdx = Math.min(rightIdx, beforeFirstNLRight);
        const window = buffer.substr(winLeftIdx, winRightIdx - winLeftIdx + 1);
        // find caret position in last line
        const caretPos = failurePos - winLeftIdx + 1;
        // augment with diagnostic information
        const diag = leftPad('', '=', winRightIdx - winLeftIdx + 1) +
            '\n' +
            err +
            '\n' +
            window +
            '\n' +
            leftPad('^', ' ', caretPos - 1) +
            '\n' +
            leftPad('', '=', winRightIdx - winLeftIdx + 1) +
            '\n';
        return diag;
    }
    Primitives.diagnosticMessage = diagnosticMessage;
    /**
     * nl matches and returns a newline.
     */
    Primitives.nl = Primitives.choice(Primitives.str('\n'))(Primitives.str('\r\n'));
    /**
     * wschars matches any whitespace char, namely
     * ' ', '\t', '\n', or '\r\n'.
     */
    const wschars = choice(satClass([' ', '\t']))(Primitives.nl);
    /**
     * ws matches zero or more of the following whitespace characters:
     * ' ', '\t', '\n', or '\r\n'
     * ws returns matched whitespace in a single CharStream result.
     * Note: ws NEVER fails
     */
    Primitives.ws = function* (istream) {
        const o = yield* many(wschars)(istream);
        switch (o.tag) {
            case 'success':
                return new Success(o.inputstream, CharStream.concat(o.result));
            case 'failure':
                return o;
        }
    };
    /**
     * ws1 matches one or more of the following whitespace characters:
     * ' ', '\t', '\n', or '\r\n'
     * ws1 returns matched whitespace in a single CharStream result.
     */
    Primitives.ws1 = function* (istream) {
        const o = yield* many1(wschars)(istream);
        switch (o.tag) {
            case 'success':
                return new Success(o.inputstream, CharStream.concat(o.result));
            case 'failure':
                return o;
        }
    };
    function groupBy(list, keyGetter) {
        const m = new Map();
        list.forEach(elem => {
            const key = keyGetter(elem);
            if (!m.has(key)) {
                m.set(key, []);
            }
            const collection = m.get(key);
            collection.push(elem);
        });
        return m;
    }
    /**
     * Match any of given alternatives in the given array of strings. Matches
     * longest-first and, where length is the same, lexicographically first.
     * @param strs An array of acceptable strings.
     */
    function strSat(strs) {
        // sort strings first by length, and then lexicograpically;
        // slice() called here so as not to modify original array
        const smap = groupBy(strs, s => s.length);
        const sizes = [];
        // find size classes;
        // also sort each set of equivalent-length values
        smap.forEach((vals, key, _m) => {
            sizes.push(key);
            vals.sort();
        });
        sizes.sort().reverse();
        return function* (istream) {
            // start with the smallest size class
            for (let peekIndex = 0; peekIndex < sizes.length; peekIndex++) {
                // for each size class, try matching all of
                // the strings; if one is found, return the
                // appropriate CharStream; if not, fail.
                const peek = istream.peek(sizes[peekIndex]);
                const tail = istream.seek(sizes[peekIndex]);
                const candidates = smap.get(sizes[peekIndex]);
                for (let cIndex = 0; cIndex < candidates.length; cIndex++) {
                    if (candidates[cIndex] === peek.toString()) {
                        return new Success(tail, peek);
                    }
                }
            }
            return new Failure(istream, istream.startpos);
        };
    }
    Primitives.strSat = strSat;
    /**
     * An optimized parser that seeks the input until the given predicate does not
     * return true.  On success, it returns the matching CharStream.
     * @param pred A function that returns true if the given character should be accepted.
     */
    function matchWhile(pred) {
        return function* (istream) {
            const rem = istream.seekWhile(pred);
            const diff = rem.startpos - istream.startpos;
            if (diff == 0) {
                // no input consumed
                return new Failure(istream, istream.startpos);
            }
            else {
                const match = istream.peek(diff);
                return yield* result(match)(rem);
            }
        };
    }
    Primitives.matchWhile = matchWhile;
    /**
     * An optimized parser that seeks the input until the given predicate does not
     * return true.  On success, it returns the matching CharStream.
     * @param pred A function that returns true if the given character code should be accepted.
     */
    function matchWhileCharCode(pred) {
        return function* (istream) {
            const [match, rem] = istream.seekWhileCharCode(pred);
            if (match.startpos == match.endpos) {
                // no input consumed
                return new Failure(istream, istream.startpos);
            }
            return yield* result(match)(rem);
        };
    }
    Primitives.matchWhileCharCode = matchWhileCharCode;
})(Primitives = exports.Primitives || (exports.Primitives = {}));
//# sourceMappingURL=primitives.js.map